/**
 * Copyright (c) 2008-2023, MOVES Institute, Naval Postgraduate School (NPS). All rights reserved.
 * This work is provided under a BSD-style open-source license, see project
 * <a href="https://savage.nps.edu/opendis7-java/license.html" target="_blank">license.html</a> and <a href="https://savage.nps.edu/opendis7-java/license.txt" target="_blank">license.txt</a>
 */
// header autogenerated using string template dis7javalicense.txt



package edu.nps.moves.dis7.pdus;

import java.util.*;
import java.io.*;
import edu.nps.moves.dis7.enumerations.*;

/**
 * Jamming technique. Section 6.2.49, uid 284
 * @see <a href="https://ieeexplore.ieee.org/document/6387564" target="_blank">IEEE Std 1278.1-2012, IEEE Standard for Distributed Interactive Simulation - Application Protocols</a> 
 */
public class JammingTechnique extends Object implements Serializable
{
   /** kind is an undescribed parameter... */
   protected byte kind;

   /** category is an undescribed parameter... */
   protected byte category;

   /** subCategory is an undescribed parameter... */
   protected byte subCategory;

   /** specific is an undescribed parameter... */
   protected byte specific;


/** Constructor creates and configures a new instance object */
 public JammingTechnique()
 {
 }

  /**
   * Returns size of this serialized (marshalled) object in bytes
   * @see <a href="https://en.wikipedia.org/wiki/Marshalling_(computer_science)" target="_blank">https://en.wikipedia.org/wiki/Marshalling_(computer_science)</a>
   * @return serialized size in bytes
   */
public synchronized int getMarshalledSize()
{
   int marshalSize = 0; 

   marshalSize += 1;  // kind
   marshalSize += 1;  // category
   marshalSize += 1;  // subCategory
   marshalSize += 1;  // specific

   return marshalSize;
}


/** Setter for {@link JammingTechnique#kind}
  * @param pKind new value of interest
  * @return same object to permit progressive setters */
public synchronized JammingTechnique setKind(byte pKind)
{
    kind = pKind;
    return this;
}
/** Utility setter for {@link JammingTechnique#kind}
  * @param pKind new value of interest
  * @return same object to permit progressive setters */
public synchronized JammingTechnique setKind(int pKind){
    kind = (byte) pKind;
    return this;
}
/** Getter for {@link JammingTechnique#kind}
  * @return value of interest */
public byte getKind()
{
    return kind; 
}

/** Setter for {@link JammingTechnique#category}
  * @param pCategory new value of interest
  * @return same object to permit progressive setters */
public synchronized JammingTechnique setCategory(byte pCategory)
{
    category = pCategory;
    return this;
}
/** Utility setter for {@link JammingTechnique#category}
  * @param pCategory new value of interest
  * @return same object to permit progressive setters */
public synchronized JammingTechnique setCategory(int pCategory){
    category = (byte) pCategory;
    return this;
}
/** Getter for {@link JammingTechnique#category}
  * @return value of interest */
public byte getCategory()
{
    return category; 
}

/** Setter for {@link JammingTechnique#subCategory}
  * @param pSubCategory new value of interest
  * @return same object to permit progressive setters */
public synchronized JammingTechnique setSubCategory(byte pSubCategory)
{
    subCategory = pSubCategory;
    return this;
}
/** Utility setter for {@link JammingTechnique#subCategory}
  * @param pSubCategory new value of interest
  * @return same object to permit progressive setters */
public synchronized JammingTechnique setSubCategory(int pSubCategory){
    subCategory = (byte) pSubCategory;
    return this;
}
/** Getter for {@link JammingTechnique#subCategory}
  * @return value of interest */
public byte getSubCategory()
{
    return subCategory; 
}

/** Setter for {@link JammingTechnique#specific}
  * @param pSpecific new value of interest
  * @return same object to permit progressive setters */
public synchronized JammingTechnique setSpecific(byte pSpecific)
{
    specific = pSpecific;
    return this;
}
/** Utility setter for {@link JammingTechnique#specific}
  * @param pSpecific new value of interest
  * @return same object to permit progressive setters */
public synchronized JammingTechnique setSpecific(int pSpecific){
    specific = (byte) pSpecific;
    return this;
}
/** Getter for {@link JammingTechnique#specific}
  * @return value of interest */
public byte getSpecific()
{
    return specific; 
}

/**
 * Serializes an object to a DataOutputStream.
 * @throws java.lang.Exception if something goes wrong
 * @see java.io.DataOutputStream
 * @param dos the OutputStream
 */
public synchronized void marshal(DataOutputStream dos) throws Exception
{
    try 
    {
       dos.writeByte(kind);
       dos.writeByte(category);
       dos.writeByte(subCategory);
       dos.writeByte(specific);
    }
    catch(Exception e)
    {
      System.err.println(e);
    }
}

/**
 * Deserializes an object from a DataInputStream.
 * @throws java.lang.Exception if something goes wrong
 * @see java.io.DataInputStream
 * @see <a href="https://en.wikipedia.org/wiki/Marshalling_(computer_science)" target="_blank">https://en.wikipedia.org/wiki/Marshalling_(computer_science)</a>
 * @param dis the InputStream
 * @return marshalled serialized size in bytes
 */
public synchronized int unmarshal(DataInputStream dis) throws Exception
{
    int uPosition = 0;
    try 
    {
        kind = (byte)dis.readUnsignedByte();
        uPosition += 1;
        category = (byte)dis.readUnsignedByte();
        uPosition += 1;
        subCategory = (byte)dis.readUnsignedByte();
        uPosition += 1;
        specific = (byte)dis.readUnsignedByte();
        uPosition += 1;
    }
    catch(Exception e)
    { 
      System.err.println(e); 
    }
    return getMarshalledSize();
}

/**
 * Packs an object into the ByteBuffer.
 * @throws java.nio.BufferOverflowException if byteBuffer is too small
 * @throws java.nio.ReadOnlyBufferException if byteBuffer is read only
 * @see java.nio.ByteBuffer
 * @param byteBuffer The ByteBuffer at the position to begin writing
 * @throws Exception ByteBuffer-generated exception
 */
public synchronized void marshal(java.nio.ByteBuffer byteBuffer) throws Exception
{
   byteBuffer.put( (byte)kind);
   byteBuffer.put( (byte)category);
   byteBuffer.put( (byte)subCategory);
   byteBuffer.put( (byte)specific);
}

/**
 * Unpacks a Pdu from the underlying data.
 * @throws java.nio.BufferUnderflowException if byteBuffer is too small
 * @see java.nio.ByteBuffer
 * @see <a href="https://en.wikipedia.org/wiki/Marshalling_(computer_science)" target="_blank">https://en.wikipedia.org/wiki/Marshalling_(computer_science)</a>
 * @param byteBuffer The ByteBuffer at the position to begin reading
 * @return marshalled serialized size in bytes
 * @throws Exception ByteBuffer-generated exception
 */
public synchronized int unmarshal(java.nio.ByteBuffer byteBuffer) throws Exception
{
    try
    {
        // attribute kind marked as not serialized
        kind = (byte)(byteBuffer.get() & 0xFF);
        // attribute category marked as not serialized
        category = (byte)(byteBuffer.get() & 0xFF);
        // attribute subCategory marked as not serialized
        subCategory = (byte)(byteBuffer.get() & 0xFF);
        // attribute specific marked as not serialized
        specific = (byte)(byteBuffer.get() & 0xFF);
    }
    catch (java.nio.BufferUnderflowException bue)
    {
        System.err.println("*** buffer underflow error while unmarshalling " + this.getClass().getName());
    }
    return getMarshalledSize();
}

 /*
  * Override of default equals method.  Calls equalsImpl() for content comparison.
  */
@Override
 public synchronized boolean equals(Object obj)
 {
    if(this == obj)
      return true;

    if(obj == null)
       return false;

    if(!getClass().isAssignableFrom(obj.getClass())) //if(getClass() != obj.getClass())
        return false;

    return equalsImpl(obj);
 }

 /**
  * Compare all fields that contribute to the state, ignoring
  * transient and static fields, for <code>this</code> and the supplied object
  * @param obj the object to compare to
  * @return true if the objects are equal, false otherwise.
  */
 public synchronized boolean equalsImpl(Object obj)
 {
     final JammingTechnique rhs = (JammingTechnique)obj;

     if( ! (kind == rhs.kind)) return false;
     if( ! (category == rhs.category)) return false;
     if( ! (subCategory == rhs.subCategory)) return false;
     if( ! (specific == rhs.specific)) return false;
    return true;
 }

 @Override
 public synchronized String toString()
 {
    StringBuilder sb  = new StringBuilder();
    StringBuilder sb2 = new StringBuilder();
    sb.append(getClass().getSimpleName());
    sb.append(" kind:").append(kind); // writeOneToString
    sb.append(" category:").append(category); // writeOneToString
    sb.append(" subCategory:").append(subCategory); // writeOneToString
    sb.append(" specific:").append(specific); // writeOneToString

   return sb.toString();
 }

 @Override
 public int hashCode()
 {
	 return Objects.hash(this.kind,
	                     this.category,
	                     this.subCategory,
	                     this.specific);
 }
} // end of JammingTechnique
